"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addBackwardPolyAssociation = void 0;
const utils_1 = require("./utils");
exports.addBackwardPolyAssociation = (builder, option) => {
    // const { pgSimpleCollections } = option;
    // const hasConnections = pgSimpleCollections !== 'only';
    builder.hook('GraphQLObjectType:fields', (fields, build, context) => {
        const { extend, inflection, pgOmit: omit, pgPolymorphicClassAndTargetModels, } = build;
        const { scope: { isPgRowType, pgIntrospection: table }, fieldWithHooks, } = context;
        if (!isPgRowType || !table || table.kind !== 'class') {
            return fields;
        }
        const tablePKey = utils_1.getPrimaryKey(table);
        if (!tablePKey) {
            return fields;
        }
        utils_1.validatePrerequisit(build);
        const modelName = inflection.tableType(table);
        // Find  all the forward relations with polymorphic
        const isConnection = true;
        const backwardPolyAssociation = pgPolymorphicClassAndTargetModels
            .filter(con => con.to.find(c => c.pgClass.id === table.id))
            .reduce((memo, currentPoly) => {
            // const { name } = currentPoly;
            const foreignTable = currentPoly.from;
            if (omit(foreignTable, 'read')) {
                return memo;
            }
            const isForeignKeyUnique = utils_1.polyForeignKeyUnique(build, foreignTable, currentPoly);
            const fieldName = inflection.backwardRelationByPolymorphic(foreignTable, currentPoly, isForeignKeyUnique);
            const fieldFunction = utils_1.generateFieldWithHookFunc(build, foreignTable, (qBuilder, innerBuilder) => {
                innerBuilder.where(utils_1.polymorphicCondition(build, currentPoly, innerBuilder, qBuilder, modelName, tablePKey));
                if (!isForeignKeyUnique) {
                    utils_1.ensureBuilderUniqueOrder(build, innerBuilder, foreignTable);
                }
            }, isForeignKeyUnique, isConnection);
            return Object.assign(Object.assign({}, memo), { [fieldName]: fieldWithHooks(fieldName, fieldFunction, {
                    isPgFieldConnection: isConnection,
                    isPgFieldSimpleCollection: !isConnection,
                    isPgBackwardPolymorphicField: true,
                    pgFieldIntrospection: foreignTable,
                }) });
        }, {});
        return extend(fields, backwardPolyAssociation);
    });
};
//# sourceMappingURL=add_backward_poly_association_plugin.js.map