"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addModelTableMappingPlugin = void 0;
exports.addModelTableMappingPlugin = (builder, options) => {
    const { pgSchemas = [] } = options;
    builder.hook('build', (build) => {
        const { inflection, pgIntrospectionResultsByKind: { procedure, class: pgClasses }, } = build;
        const fieldToDBMap = pgClasses.reduce((acc, cur) => {
            // Only build the map for the included schema.
            // Also this mapping behave unexpectedly if there are tables with same name in different
            // Schema used in the postgraphile
            if (!pgSchemas.includes(cur.namespaceName) ||
                cur.namespaceName === 'pg_catalog' ||
                cur.namespaceName === 'information_schema') {
                // skipt it
                return acc;
            }
            const procedureAttriutesMap = procedure
                .filter(p => p.name.startsWith(`${cur.name}_`))
                .reduce((a, c) => {
                // Should probably use inflection
                const name = inflection.computedColumn(c.name.substr(cur.name.length + 1), c, cur);
                return Object.assign(Object.assign({}, a), { [name]: c });
            }, {});
            // The Model Name points to this pg object
            return Object.assign(Object.assign({}, acc), { [inflection.tableType(cur)]: {
                    name: cur.name,
                    id: cur.id,
                    attributesMap: cur.attributes.reduce((allAtt, curA) => {
                        return Object.assign(Object.assign({}, allAtt), { [inflection.column(curA)]: curA });
                    }, procedureAttriutesMap),
                } });
        }, {});
        return build.extend(build, { mapFieldToPgTable: fieldToDBMap });
    });
};
//# sourceMappingURL=define_model_to_table_map_plugin.js.map