"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addForwardPolyRelationFilter = exports.getTableFilterType = exports.generateFilterResolveFunc = void 0;
const utils_1 = require("./utils");
exports.generateFilterResolveFunc = (build, poly, sourceTable, foreignTable, foreignIsPoly) => {
    const { pgSql: sql, inflection, connectionFilterResolve, } = build;
    const foreignTableTypeName = inflection.tableType(foreignTable);
    const sourceTableTypeName = inflection.tableType(sourceTable);
    const foreignTableAlias = sql.identifier(Symbol());
    const foreignTableFilterTypeName = inflection.filterType(foreignTableTypeName);
    const sqlIdentifier = sql.identifier(foreignTable.namespace.name, foreignTable.name);
    const resolve = ({ sourceAlias, fieldValue, queryBuilder, }) => {
        if (fieldValue == null)
            return null;
        let sqlKeysMatch = '';
        if (foreignIsPoly) {
            // If foreign table is poly table, then we need to switch the alias
            const pKey = utils_1.getPrimaryKey(sourceTable);
            sqlKeysMatch = utils_1.polySqlKeyMatch(build, foreignTableAlias, sourceAlias, pKey, sourceTableTypeName, poly);
        }
        else {
            // If foreign table is not poly, means it is a regular table.
            const pKey = utils_1.getPrimaryKey(foreignTable);
            sqlKeysMatch = utils_1.polySqlKeyMatch(build, sourceAlias, foreignTableAlias, pKey, foreignTableTypeName, poly);
        }
        const sqlFragment = connectionFilterResolve(fieldValue, foreignTableAlias, foreignTableFilterTypeName, queryBuilder);
        return sqlFragment == null
            ? null
            : sql.query ` exists(
        select 1 from ${sqlIdentifier} as ${foreignTableAlias}
        where ${sqlKeysMatch} and
          (${sqlFragment})
      )`;
    };
    return resolve;
};
exports.getTableFilterType = (build, table) => {
    const { newWithHooks, inflection, connectionFilterType } = build;
    const foreignTableTypeName = inflection.tableType(table);
    const foreignTableFilterTypeName = inflection.filterType(foreignTableTypeName);
    const ForeignTableFilterType = connectionFilterType(newWithHooks, foreignTableFilterTypeName, table, foreignTableTypeName);
    return ForeignTableFilterType;
};
exports.addForwardPolyRelationFilter = (builder) => {
    builder.hook('GraphQLInputObjectType:fields', (fields, build, context) => {
        const { extend, inflection, connectionFilterTypesByTypeName, connectionFilterRegisterResolver, pgPolymorphicClassAndTargetModels = [], } = build;
        const { fieldWithHooks, scope: { pgIntrospection: table, isPgConnectionFilter }, Self, } = context;
        if (!isPgConnectionFilter || table.kind !== 'class')
            return fields;
        utils_1.validatePrerequisit(build);
        connectionFilterTypesByTypeName[Self.name] = Self;
        // Iterate the pgPolymorphic constraints and find the ones that are relavent to this table
        const forwardPolyRelationSpecs = pgPolymorphicClassAndTargetModels
            .filter(con => con.from.id === table.id)
            .reduce((acc, currentPoly) => {
            const toReturn = currentPoly.to.reduce((memo, curForeignTable) => {
                const { pgClass: foreignTable } = curForeignTable;
                const fName = inflection.forwardRelationByPolymorphic(foreignTable, currentPoly.name);
                const ForeignTableFilterType = exports.getTableFilterType(build, foreignTable);
                if (!ForeignTableFilterType)
                    return memo;
                // Field
                const resolve = exports.generateFilterResolveFunc(build, currentPoly, table, foreignTable, false);
                connectionFilterRegisterResolver(Self.name, fName, resolve);
                return extend(memo, {
                    [fName]: fieldWithHooks(fName, {
                        description: `Filter by the object’s \`${fName}\` polymorphic relation.`,
                        type: ForeignTableFilterType,
                    }, {
                        isPgConnectionFilterField: true,
                    }),
                });
            }, {});
            return extend(acc, toReturn);
        }, {});
        return extend(fields, forwardPolyRelationSpecs);
    });
};
//# sourceMappingURL=pgConnectionArgFilterForwardPolyRelationPlugin.js.map